/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.sftp;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

import org.ow2.petals.bc.sftp.connection.SFTPConnectionInfo;
import org.ow2.petals.bc.sftp.connection.SftpClientWrapper;

import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.SshClient;
import com.sshtools.j2ssh.authentication.AuthenticationProtocolState;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.transport.IgnoreHostKeyVerification;

/**
 * Utility class that holds the SSH connection pool for the component.
 * 
 * @author Charles Casadei - eBM WebSourcing
 * @author Mathieu CARROLLE - PetalsLink"
 */
public final class SFTPClientPool {

    /**
     * The actual SFTP connections.
     */
    private static Map<String, SftpClientWrapper> sftpClients = new ConcurrentHashMap<String, SftpClientWrapper>();

    /**
     * Get an SftpClient from an already registered SSH session or a new one.
     * 
     * Sftp client are instantiated using SshClient, which are stored in a Map,
     * which keys are <code>host+port+login+password</code>. If a client
     * retrieved from the Map is disconnected, another one is created and
     * connected.
     * 
     * @param info
     *            connection informations
     * @return a connected SftpClient
     * @throws SshAuthenticationException
     *             if the authentication failed or if the opened connection does
     *             not give access to a valid SFTP client
     * @throws SftpClientPoolException
     *             if no SFTP client can be retrieved from the SSH connection &
     *             the client pool was not able to remove the SSH client
     *             properly
     */
    public synchronized static final SftpClient getSFTPClient(final SFTPConnectionInfo info, Logger logger)
            throws SshAuthenticationException, SftpClientPoolException {

        final String key = info.getKey();

        // Retrieve the SSH client associated to the previous key
        final SftpClientWrapper clientWrapper = sftpClients.get(key);
        if (clientWrapper != null) {
            final SftpClient client = clientWrapper.getSftpClient();
            if (client.isClosed()) {
                cleanSFTPClient(info);
            } else {
                return client;
            }
        }

        return createSFTPClient(info, logger);
    }

    /**
     * Clean the SFTP Client.
     * 
     * @param info
     */
    public static final void cleanSFTPClient(final SFTPConnectionInfo info) {
        final SftpClientWrapper sftpClientWrapper = sftpClients.remove(info.getKey());

        if (sftpClientWrapper != null) {
            final SshClient sshClient = sftpClientWrapper.getSshClient();
            if (sshClient != null) {
                try {
                    sftpClientWrapper.getSftpClient().quit();
                } catch (IOException e) {
                    // do nothing
                }
                sshClient.disconnect();
            }
        }
    }

    /**
     * Clean the SFTP Client.
     * 
     * @param info
     */
    public static final void cleanSFTPClient(final SftpClientWrapper sftpClientWrapper) {
        for (String key : sftpClients.keySet()) {
            // use == and not equals, the object must bne the same instance to
            // delete it
            if (sftpClients.get(key) == sftpClientWrapper) {
                sftpClients.remove(key);
                break;
            }
        }
    }

    /**
     * Return a connected SshClient.
     * 
     * Note : If this method fails the SSH connection is automatically closed.
     * 
     * @param info
     *            required informations to create a new SSH connection
     * @return a connected SftpClient
     * @throws SshAuthenticationException
     *             raised only if the authentication process failed, the SSH
     *             client is automatically closed. Clients of this method won't
     *             have to do it themselves.
     * @throws SftpClientPoolException
     */
    private static final SftpClient createSFTPClient(final SFTPConnectionInfo info, final Logger logger)
            throws SshAuthenticationException, SftpClientPoolException {

        // Configure J2SSH (This will attempt to install the bouncycastle
        // provider under jdk 1.3.1)
        try {
            ConfigurationLoader.initialize(false);
        } catch (com.sshtools.j2ssh.configuration.ConfigurationException ex) {
            // to avoid name clash
            throw new SshAuthenticationException("Can't initialize the SSH client : ", ex);
        }
        // At this point :
        // - Bouncy castle may have been installed

        // Make a client connection

        final SshClient sshClient = new SshClient();

        SftpClientWrapper sftpClientWrapper = new SftpClientWrapper(sshClient, logger);
        if (info.getMaxIdleTime() != 0) {
            sshClient.setSocketTimeout(new Long((info.getMaxIdleTime() * 60000)).intValue());
            sshClient.addEventHandler(sftpClientWrapper);
        }

        try {
            // Try to open a connection to the host
            try {
                sshClient
                        .connect(info.getServer(), info.getPort(), new IgnoreHostKeyVerification());
            } catch (IOException ex) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't open a connection to : ");
                errorSB.append(info);
                errorSB.append(" : ");
                throw new SshAuthenticationException(errorSB.toString(), ex);
            }

            // Retrieve the authentication informations
            final SshAuthenticationClient sshAuthenticationClient;
            try {
                sshAuthenticationClient = info.getSshAuthentication().getAuthenticationClient();
            } catch (SshAuthenticationException ex) {
                throw ex;
            }

            // Try the authentication
            int authenticationResult = 0;
            try {
                authenticationResult = sshClient.authenticate(sshAuthenticationClient);
            } catch (IOException ex) {
                throw new SshAuthenticationException("Error during authentication : ", ex);
            }

            if (authenticationResult != AuthenticationProtocolState.COMPLETE) {
                throw new SshAuthenticationException("Authentication failed");
            }
            // At this point :
            // - SSH client is connected
            // - SSH client is authenticated

        } catch (SshAuthenticationException ex) {
            // If the client is connected > close the connection
            if (sshClient.isConnected()) {
                sshClient.disconnect();
            }
            throw ex;
        }

        final SftpClient sftpClient;
        try {
            sftpClient = sshClient.openSftpClient();

            if (info.getDirectory().length() > 0) {
                sftpClient.cd(info.getDirectory());
            }
        } catch (IOException e) {
            throw new SftpClientPoolException(e.getMessage(), e);
        }

        sftpClientWrapper.setSftpClient(sftpClient);
        SFTPClientPool.sftpClients.put(info.getKey(), sftpClientWrapper);

        return sftpClient;
    }
}
