/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.sftp.connection;

import org.ow2.petals.bc.sftp.ConfigurationException;
import org.ow2.petals.bc.sftp.SFTPConstants;
import org.ow2.petals.bc.sftp.authentication.SshAuthentication;
import org.ow2.petals.bc.sftp.authentication.SshPasswordAuthentication;
import org.ow2.petals.bc.sftp.authentication.SshPublicKeyAuthentication;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Node;

/**
 * Helper class which helps (sic) building SFTPConnectionInfo instance using the
 * values given in the SU ConfigurationExtensions or using the configuration
 * node given in a JBI message.
 * 
 * @author Charles Casadei - eBM WebSourcing
 */
public class SFTPConnectionInfoBuilder {

    /**
     * Builds a new instance of SFTPConnection info using the informations from
     * the given node. This allow the SFTP component to be parameterized at
     * runtime.
     * 
     * @param node
     *            the service unit configuration
     * @return a new SFTPConnectionInfo instance
     * @throws ConfigurationException
     *             if the service configuration is not valid
     */
    public static SFTPConnectionInfo buildSFTPConnectionInfo(Node node)
            throws ConfigurationException {

        Node hostNode = null;
        Node portNode = null;
        Node userNode = null;
        Node passwordNode = null;
        Node privateKeyNode = null;
        Node passphraseNode = null;
        Node directoryNode = null;
        Node maxIdleTimeNode = null;
        Node overwrite = null;
        SshAuthentication sshAuthentication = null;
        SFTPConnectionInfo result = null;

        // Retrieve the configuration nodes from the given "root" node
        hostNode = XMLUtil.findChild(node, SFTPConstants.SFTP_SERVER, null, false);
        portNode = XMLUtil.findChild(node, SFTPConstants.SFTP_PORT, null, false);
        userNode = XMLUtil.findChild(node, SFTPConstants.SFTP_USER, null, false);
        passwordNode = XMLUtil.findChild(node, SFTPConstants.SFTP_PASSWORD, null, false);
        privateKeyNode = XMLUtil.findChild(node, SFTPConstants.SFTP_PRIVATE_KEY, null, false);
        passphraseNode = XMLUtil.findChild(node, SFTPConstants.SFTP_PASSPHRASE, null, false);
        directoryNode = XMLUtil.findChild(node, SFTPConstants.SFTP_FOLDER, null, false);
        maxIdleTimeNode = XMLUtil.findChild(node, SFTPConstants.SFTP_MAX_IDLE_TIME, null, false);
        overwrite = XMLUtil.findChild(node, SFTPConstants.SFTP_OVERWRITE, null, false);
        // If the user node is empty
        if (userNode == null) {
            throw new ConfigurationException("Missing configuration element : "
                    + SFTPConstants.SFTP_USER);
        }

        // Checks if there's only one authentication mode (ie if password => no
        // private key & no passphrase
        if (((passwordNode != null) && (privateKeyNode != null))
                || ((passwordNode != null) && (passphraseNode != null))) {
            throw new ConfigurationException(
                    "Can't use both password authentication and private key authentication");
        }

        // A passphrase must match a private key...if no private key -> bang :-)
        if ((passphraseNode != null) && (privateKeyNode == null)) {
            throw new ConfigurationException("A passphrase with no matching private key was found");
        }

        // If the authentication method is : user / password
        if (passwordNode != null) {
            sshAuthentication = new SshPasswordAuthentication(userNode.getTextContent(),
                    passwordNode.getTextContent());
        }
        // Else if the authentication method is user / privatekey / passphrase
        else if (privateKeyNode != null) {
            sshAuthentication = new SshPublicKeyAuthentication(userNode.getTextContent(),
                    privateKeyNode.getTextContent(),
                    ((passphraseNode != null) ? passphraseNode.getTextContent() : null));
        } else {
            throw new ConfigurationException("No authentications information found");
        }

        // Builds a SFTPConnectionInfo object from the previous nodes
        result = new SFTPConnectionInfo(((hostNode != null) ? hostNode.getTextContent() : null),
                ((portNode != null) ? portNode.getTextContent() : null), sshAuthentication,
                ((directoryNode != null) ? directoryNode.getTextContent() : null),
                ((maxIdleTimeNode != null) ? Long.parseLong(maxIdleTimeNode.getTextContent())
                        : SFTPConstants.DEFAULT_SSH_MAX_IDLE_TIME),
                overwrite != null ? Boolean.parseBoolean(overwrite.getTextContent()) : false);

        return result;
    }

    /**
     * TODO: move that in a Validator extension Builds a new instance of
     * SFTPConnectionInfo using the information given in the service unit
     * descriptor.
     * 
     * @param extensions
     *            the service unit configuration
     * @return a new SFTPConnectionInfo instance
     * @throws ConfigurationException
     *             if the service configuration is not valid
     */
    public static SFTPConnectionInfo buildSFTPConnectionInfo(
            final ConfigurationExtensions extensions) throws ConfigurationException {

        SshAuthentication sshAuthentication = null;

        final String host = extensions.get(SFTPConstants.SFTP_SERVER);
        final String port = extensions.get(SFTPConstants.SFTP_PORT);
        final String user = extensions.get(SFTPConstants.SFTP_USER);
        final String password = extensions.get(SFTPConstants.SFTP_PASSWORD);
        final String privateKey = extensions.get(SFTPConstants.SFTP_PRIVATE_KEY);
        final String passphrase = extensions.get(SFTPConstants.SFTP_PASSPHRASE);
        final String directory = extensions.get(SFTPConstants.SFTP_FOLDER);
        final String maxIdleTime = extensions.get(SFTPConstants.SFTP_MAX_IDLE_TIME);
        final boolean overwrite = Boolean
                .parseBoolean(extensions.get(SFTPConstants.SFTP_OVERWRITE));

        // Checks if there's only one authentication mode (ie if password => no
        // private key & no passphrase
        if (((password != null) && (privateKey != null))
                || ((password != null) && (passphrase != null))) {
            throw new ConfigurationException(
                    "Can't use both password authentication and private key authentication");
        }

        // A passphrase must match a private key...if no private key -> bang :-)
        if ((passphrase != null) && (privateKey == null)) {
            throw new ConfigurationException("A passphrase with no matching private key was found");
        }

        // If the authentication method is : user / password
        if (password != null) {
            sshAuthentication = new SshPasswordAuthentication(user, password);
        }
        // Else if the authentication method is user / privatekey / passphrase
        else if (privateKey != null) {
            sshAuthentication = new SshPublicKeyAuthentication(user, privateKey, passphrase);
        } else {
            throw new ConfigurationException("No authentication informations found");
        }

        return new SFTPConnectionInfo(host, port, sshAuthentication, directory,
                ((maxIdleTime != null) ? Long.parseLong(maxIdleTime)
                        : SFTPConstants.DEFAULT_SSH_MAX_IDLE_TIME), overwrite);
    }
}
